Imports System.Windows.Forms
Imports System.ComponentModel

#Region "Enumeration FilterModus"
''' <summary>
''' Definiert die verschiedenen Filtermodi, die vom Steuerelement
''' FilterTextBox untersttzt werden
''' </summary>
Public Enum FilterModus
   ''' <summary>
   ''' Kein Filter aktiv, FilterTextBox verhlt sich wie TextBox.
   ''' </summary>
    None

   ''' <summary>
   ''' Nur binre Zahlen (0 und 1) erlaubt.
   ''' </summary>
   NumericBinary

   ''' <summary>
   ''' Nur Zahlen des Dezimalsystems (0 bis 9) erlaubt.
   ''' </summary>
   NumericDecimal

   ''' <summary>
   ''' Nur Zahlen und Buchstaben des Hexadezimalsystems (0 bis 9, A bis F) erlaubt.
   ''' </summary>
    NumericHexadecimal

   ''' <summary>
   ''' Nur Buchstaben (A-Z) erlaubt.
   ''' </summary>
    LettersOnly

   ''' <summary>
    ''' Benutzerdefinierten Filter verwenden
   ''' </summary>
    CustomFilter

End Enum
#End Region

#Region "Klasse FilterTextBox"
Public Class FilterTextBox : Inherits TextBox

#Region "ffentliche Eigenschaften"

   ''' <summary>
   ''' Legt den gewnschten Filtermodus fest oder ruft diesen ab.
   ''' </summary>
   <Category("Default")> _
   <Description("Legt den gewnschten Filtermodus fest oder ruft diesen ab.")> _
   Public Property FilterMode As FilterModus

   ''' <summary>
   ''' Legt fest, ob fr eine nicht erlaubte Taste ein Signalton ausgegeben wird.
   ''' </summary>
   <Category("Default")> _
   <Description("Legt fest, ob fr eine nicht erlaubte Taste ein Signalton ausgegeben wird oder ruft diesen Wert ab.")> _
   Public Property BeepOnError As Boolean

   ''' <summary>
   ''' Legt einen benutzerdefinierten Filter fest oder ruft diesen ab.
   ''' </summary>
   <Category("Default")> _
   <Description("Legt die Zeichen des benutzerdefinierten Filters fest oder ruft diese ab.")> _
   Public Property CustomFilter As String

#End Region

#Region "Nichtffentliche Eigenschaften"
   ' erlaubte Zeichen im Modus NumericBinary definieren
   Private mAllowedBinary As String = "01"

   ' erlaubte Zeichen im Modus NumericHexadecimal definieren
   Private mAllowedHexadedimal As String = "0123456789abcdefABCDEF"
#End Region

#Region "Konstruktor"
   ''' <summary>
   ''' Erstellt ein neues Steuerelement des Typs FilterTextBox
   ''' </summary>
   Public Sub New()
      ' Standardwerte festlegen
      Me.BeepOnError = False
      Me.FilterMode = FilterModus.LettersOnly
   End Sub
#End Region

 

   Private Function isCharAllowed(ByVal keyChar As Char) As Boolean

      If Char.IsControl(keyChar) Then
         Return True
      End If

      Select Case Me.FilterMode
         Case FilterModus.NumericBinary
            If InStr(mAllowedBinary, keyChar) > 0 Then
               Return True
            Else
               Return False
            End If

         Case FilterModus.NumericHexadecimal
            If InStr(mAllowedHexadedimal, keyChar) > 0 Then
               Return True
            Else
               Return False
            End If

         Case FilterModus.CustomFilter
            If InStr(Me.CustomFilter, keyChar) > 0 Then
               Return True
            Else
               Return False
            End If

         Case FilterModus.NumericDecimal
            If Char.IsDigit(keyChar) Then
               Return True
            Else
               Return False
            End If

         Case FilterModus.LettersOnly
            If Char.IsLetter(keyChar) Then
               Return True
            Else
               Return False
            End If

         Case Else
            Return False
      End Select
   End Function


   Protected Overrides Sub OnKeyPress(ByVal e As System.Windows.Forms.KeyPressEventArgs)
      ' Kein Filtermodus aktiv, alle Arbeit von TextBox.OnKeyPress erledigen lassen
      If Me.FilterMode = FilterModus.None Then
         e.Handled = False
         MyBase.OnKeyPress(e)
         Return
      End If

      ' Kein erlaubtes Zeichen?
      ' e.Handled auf True setzen, damit die Taste nicht weiter verarbeitet wird
      If Me.isCharAllowed(e.KeyChar) = False Then
         e.Handled = True
         If Me.BeepOnError = True Then
            System.Console.Beep()
         End If
         ' Erlabutes Zeichen?
         ' Alle Arbeit von TextBox.OnKeyPress erledigen lassen.
      Else
         MyBase.OnKeyPress(e)
      End If

   End Sub


End Class
#End Region
